-- Interactive search
-- Ben Lin 5/98

class I_SEARCH inherit SEARCH_STRATEGY
	-- Interactive search - shows suggested moves (from get_open)
	-- and asks user to select one. -1 quits, 0 undoes

feature

	display_list(list : COLLECTION[COMMAND]) is
		-- Display list of suggestions
	local
		i : integer
	do
		io.put_string("Suggestions:%N")
		from i := list.lower
		until i > list.upper
		loop
			io.put_integer(i - list.lower + 1)
			io.put_string(": ")
			list.item(i).display
			i := i + 1
		end
	end

	doit(bb : BLACKBOARD) is
		-- do the search on the blackboard bb
	local
		i : INTEGER
		command_list : LINK2_LIST[COMMAND]
		history : LINK2_LIST[COMMAND]
		valid_input : BOOLEAN
		finished : BOOLEAN
	do
		!!history.make
		from
			if bb.finished then
				finished := true
			end
		until finished
		loop

			command_list := bb.get_open
			display_list(command_list)
			bb.display
			from valid_input := false
			until valid_input
			loop
				io.put_string("Choice? ")
				io.read_integer
				i := io.last_integer
				if i >= -1 and i <= command_list.count then
					valid_input := true
				end
			end

			if i = 0 then
				if not history.empty then
					history.last.unexecute
					history.remove_last
				else
					io.put_string("Can't undo%N")
				end
			else
				if i = -1 then
					finished := true
				else
					history.add_last(command_list.item(i))
					command_list.item(i).execute
					if bb.finished then
						finished := true
					end
				end
			end

		end
	end

	display is
	do
		io.put_string("Interactive Search%N")
	end
end

