-- Logic problem - "Limbering Up"
-- from "Logic Problems" No. 15 (by A.J. Duncum)
-- Ben Lin 26/5/98
--
-- The "board" is made of 6 3 x 3 grids (called subboards),
-- and when displayed, is the same as the first 3 lines of the
-- grid in the book (i.e. the top 3 subboards). The rest of the
-- grid differs from the book; they are placed in different
-- positions and often reflections of those in the book. This doesn't
-- matter because only the top 3 grids are used to read off a solution.
--
-- The solution to this one is:
--		 B O W  N Q V  1 8 C
--		 e a o  S L I  0 0 C
--		 e k o  W D C  0 0
--		 c e d
--		 h s
--
--  John O x x  x x O  x x O
-- Sally x x O  O x x  O x x
-- Wilma x O x  x O x  x O x
--
-- where O is a tick and x is a cross. (They are the top 3 lines the
-- program prints.)
--
-- In other, words: John Beech from Victoria won the cross country,
-- Sally Wood from New South Wales won the 100m, and Wilma Oakes from
-- Queensland won the 800m.
--
-- This program should find this solution without having to search at all.
 
class LIMBER
creation make
feature
	pb : LP_BOARD
	size : INTEGER is 3
	sub_count : INTEGER is 4

	-- The 4 categories are:
	-- 1 = First name (John, Sally, Wilma)
	-- 2 = Last name (Beech, Oakes Wood)
	-- 3 = Place (New South Wales, Queensland, Victoria)
	-- 4 = Event (100 metres, 800 metres, Cross country)

	--ss : DFS_SEARCH
	ss : I_SEARCH

	make is
	do
		!!pb.make(sub_count, size)
		setup_clues
		!!ss
		--io.put_string("%/27/[2J")

		ss.doit(pb)
		if not pb.finished then
			io.put_string("No solution! :(%N")
		else
			io.put_string("Solution found:%N")
			pb.display
		end
	end

	setup_clues is
	do
		tick(1, 4, 1, 3)
			-- 1. John won the cross country

		cross(1, 3, 2, 2)
			-- 2. Sally is not the Queenslander

		tick(3, 4, 2 ,2)
			-- 2. The Queensland athelete won the 800m

		tick(2, 3, 3, 1)
			-- 3. Miss Wood was from NSW

		cross(1, 2, 1, 3)
			-- 3. Miss Wood => can't be John

		cross(2, 3, 2, 3)
			-- 4. Victorian not called Oakes

	end

	tick(i, j, x, y : INTEGER) is
	local
		at : ADD_TICK
	do
		!!at.make(pb, i, j, x, y)
		do_initial_command(at)
	end

	cross(i, j, x, y : INTEGER) is
	local
		ac : ADD_CROSS
	do
		!!ac.make(pb, i, j, x, y)
		do_initial_command(ac)
	end

	do_initial_command(c : COMMAND) is
	do
		if c.finished then
			io.put_string("warning: ")
			c.display
			io.put_string("already executed%N")
		elseif not c.can_execute then
			io.put_string("warning: ")
			c.display
			io.put_string(" => no solution%N")
		end
		c.execute
	end

end
