-- Logic problem - "Tricky Customers"
-- from "Logic Problems" No. 15 (by A.J. Duncum)
-- Ben Lin 28/5/98
--
-- See throws.e for another example

class TRICKY
creation make
feature
	pb : LP_BOARD
	size : INTEGER is 5
	sub_count : INTEGER is 5

	-- The 5 categories are:
	-- 1 = Month (March, April, June, July, September)
	-- 2 = Date (2nd, 5th, 13th, 18th, 26th)
	-- 3 = Sucker (Archie Fotheringhay, Edward Tanqueray, Gerald Huntington,
	--		Montague Ffolliott, Rupert de Grey)
	-- 4 = Trickster (Bobby Dazzler, Charlie Wiles, Danny Sharp,
	--		Jimmy Bilk, Sam Dodgey)
	-- 5 = Trick (dud shares, fake diamonds, racing tips, roulette system,
	--		second-hand car)

	--ss : DFS_SEARCH
	ss : I_SEARCH

	make is
	do
		!!pb.make(sub_count, size)
		setup_clues
		!!ss
		ss.doit(pb)
		if not pb.finished then
			io.put_string("No solution! :(%N")
		else
			io.put_string("Solution found:%N")
			pb.display
		end
	end

	setup_clues is
	do
		-- Observers must come first
		connect(1, 3, 2, 1, 1, 4, 3, 3)
			-- 1. Archie got tricked in April iff Danny hit in June

		connect(1, 3, 4, 1, 1, 4, 5, 3)
			-- 1. Archie got tricked in July iff Danny hit in September

		-- Ticks and crosses

		tick(1, 2, 2, 3)
			-- 1. One took place on April 13th

		tick(3, 5, 1, 2)
			-- 2. Archie bought the diamonds

		cross(3, 4, 1, 3)
			-- 2. ...not from Danny Sharp

		cross(3, 1, 1, 1)
		cross(3, 1, 1, 3)
		cross(3, 1, 1, 5)
			-- 2. ...two months before, in fact

		tick(4, 5, 4, 4)
			-- 3. Jimmy Bilk had the roulette system

		cross(1, 4, 4, 4)
		cross(1, 4, 5, 4)
			-- 3. ...in the first half of the year

		tick(3, 4, 3, 1)
			-- 4. Gerald tricked by Bobby Dazzler

		tick(2, 3, 1, 2)
			-- 5. Edward tricked on the 2nd
	
		tick(2, 5, 5, 5)
			-- 5. ...second-hand car was on the 26th

		cross(4, 5, 2, 5)
		cross(4, 5, 3, 5)
			-- 5. ...not sold by a con man with 5-letter surname

		tick(1, 5, 3, 3)
			-- 6. Racing tips sold in June

		cross(1, 2, 1, 1)
		cross(1, 2, 1, 2)
			-- 6. ...March date had two digits

		cross(1, 5, 5, 1)
			-- 7. Dud shares not sold in September

		tick(2, 4, 2, 5)
			-- 8. Sam Dodgey hit on the 5th

		cross(3, 4, 4, 5)
			-- 8. ...victim not Montague
	end

	-- should go in lp_board?
	connect(i1, j1, x1, y1, i2, j2, x2, y2 : INTEGER) is
	local
		tl : TICK_LINK
		cl : CROSS_LINK
	do
		-- tick-link and cross-link both ways
		!!tl.make(i2, j2, x2, y2)
		pb.tick_observer.item(i1, j1, x1, y1).add_last(tl)
		!!cl.make(i2, j2, x2, y2)
		pb.cross_observer.item(i1, j1, x1, y1).add_last(cl)
		!!tl.make(i1, j1, x1, y1)
		pb.tick_observer.item(i2, j2, x2, y2).add_last(tl)
		!!cl.make(i1, j1, x1, y1)
		pb.cross_observer.item(i2, j2, x2, y2).add_last(cl)
	end
	
	larger_than(i1, x1, i2, x2, j : INTEGER) is
	local
		y : INTEGER
		larger : LARGER_THAN
		smaller : SMALLER_THAN
	do
		from y := 1
		until y > size
		loop
			!!larger.make(i1, x1, i2, x2, j)
			pb.tick_observer.item(i1, j, x1, y).add_last(larger)
			pb.cross_observer.item(i1, j, x1, y).add_last(larger)
			!!smaller.make(i2, x2, i1, x1, j)
			pb.tick_observer.item(i2, j, x2, y).add_last(smaller)
			pb.cross_observer.item(i2, j, x2, y).add_last(smaller)
			y := y + 1
		end
	end

	tick(i, j, x, y : INTEGER) is
	local
		at : ADD_TICK
	do
		!!at.make(pb, i, j, x, y)
		do_initial_command(at)
	end

	cross(i, j, x, y : INTEGER) is
	local
		ac : ADD_CROSS
	do
		!!ac.make(pb, i, j, x, y)
		do_initial_command(ac)
	end

	do_initial_command(c : COMMAND) is
	do
		if c.finished then
			io.put_string("warning: ")
			c.display
			io.put_string("already executed%N")
		elseif not c.can_execute then
			io.put_string("warning: ")
			c.display
			io.put_string(" => no solution%N")
		end
		c.execute
	end
end

